@@ -4,20 +4,20 @@
  * Copyright (C) 2002 Havoc Pennington
  * Copyright (C) 2002 Mathias Hasselmann
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This file is part of gnome-terminal.
  *
- * This library is distributed in the hope that it will be useful,
+ * Gnome-terminal is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Gnome-terminal is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "profile-editor.h"
@@ -30,7 +30,6 @@
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <string.h>
 #include <math.h>
-#include "simple-x-font-selector.h"
 #include "terminal-widget.h"
 
 #define BYTES_PER_LINE (terminal_widget_get_estimated_bytes_per_scrollback_line ())
@@ -67,6 +66,7 @@ struct _TerminalPaletteScheme
 };
 
 static TerminalPaletteScheme palette_schemes[] = {
+  { N_("Tango"), terminal_palette_tango },
   { N_("Linux console"), terminal_palette_linux },
   { N_("XTerm"), terminal_palette_xterm },
   { N_("Rxvt"), terminal_palette_rxvt }
@@ -118,8 +118,6 @@ static void       profile_editor_update_custom_command       (GtkWidget       *w
                                                               TerminalProfile *profile);
 static void       profile_editor_update_palette              (GtkWidget       *widget,
                                                               TerminalProfile *profile);
-static void       profile_editor_update_x_font               (GtkWidget       *widget,
-                                                              TerminalProfile *profile);
 static void       profile_editor_update_background_type      (GtkWidget       *widget,
                                                               TerminalProfile *profile);
 static void       profile_editor_update_background_image     (GtkWidget       *widget,
@@ -312,9 +310,6 @@ profile_changed (TerminalProfile           *profile,
   if (mask->palette)
     profile_editor_update_palette (editor, profile);
 
-  if (mask->x_font)
-    profile_editor_update_x_font (editor, profile);
-
   if (mask->background_type)
     profile_editor_update_background_type (editor, profile);
   
@@ -424,7 +419,7 @@ static void
 color_scheme_changed (GtkWidget       *combo_box,
                       TerminalProfile *profile)
 {
-  int i;
+  unsigned int i;
   
   i = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
   
@@ -433,7 +428,9 @@ color_scheme_changed (GtkWidget       *combo_box,
                                        &color_schemes[i].foreground,
                                        &color_schemes[i].background);
   else
-    ; /* "custom" selected, no change */
+    {
+      /* "custom" selected, no change */
+    }
 }
 
 static void
@@ -489,15 +486,6 @@ word_chars_changed (GtkWidget       *entry,
   g_free (text);
 }
 
-static void 
-font_changed (EggXFontSelector *selector, TerminalProfile *profile)
-{
-  gchar *name = egg_xfont_selector_get_font_name (selector);
-  if (name)
-    terminal_profile_set_x_font (profile, name);
-  g_free (name);
-}
-
 static void
 scrollbar_position_changed (GtkWidget       *combo_box,
                             TerminalProfile *profile)
@@ -600,7 +588,7 @@ static void
 palette_scheme_changed (GtkWidget       *combo_box,
                       TerminalProfile *profile)
 {
-  int i;
+  unsigned int i;
   
   i = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
   
@@ -608,7 +596,9 @@ palette_scheme_changed (GtkWidget       *combo_box,
     terminal_profile_set_palette (profile,
                                   palette_schemes[i].palette);
   else
-    ; /* "custom" selected, no change */
+    {
+      /* "custom" selected, no change */
+    }
 }
 
 static void
@@ -659,6 +649,11 @@ background_image_changed (GtkWidget       *entry,
   char *text;
   
   text = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (entry));
+
+  if (text == NULL)
+    {
+      return;
+    }
   
   terminal_profile_set_background_image_file (profile, text);
 
@@ -831,12 +826,15 @@ create_preview_pixbuf (const gchar *file)
       gchar *mime_type;
 
       mime_type = (gchar *)gnome_vfs_get_mime_type (file);
-      thumbs = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
+      if (mime_type != NULL) {
+        thumbs = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
 
-      pixbuf = gnome_thumbnail_factory_generate_thumbnail (thumbs,
-                                                           file,
-                                                           mime_type);
-      g_free (mime_type);
+        pixbuf = gnome_thumbnail_factory_generate_thumbnail (thumbs,
+                                                             file,
+                                                             mime_type);
+        g_free (mime_type);
+        g_object_unref (thumbs);
+      }
     }
   }				
   return pixbuf;
@@ -928,6 +926,7 @@ terminal_profile_edit (TerminalProfile *profile,
       double num1, num2;
       gint i;
       GtkSizeGroup *size_group;
+      GtkWidget *font_label;
 
       xml = terminal_util_load_glade_file (TERM_GLADE_FILE,
                                            "profile-editor-dialog",
@@ -1201,7 +1200,7 @@ terminal_profile_edit (TerminalProfile *profile,
 	  g_free (t);
 
 	  t = g_strdup_printf (_("Palette entry %d"), i+1);
-	  gtk_tooltips_set_tip (gtk_tooltips_data_get(w)->tooltips, w, t, NULL);
+          gtk_widget_set_tooltip_text (w, t);
 	  g_free (t);
 
 	  g_object_set_data (G_OBJECT (w),
@@ -1221,67 +1220,39 @@ terminal_profile_edit (TerminalProfile *profile,
 
       w = glade_xml_get_widget (xml, "font-hbox");
 
-      if (terminal_widget_supports_pango_fonts ())
-        {
-          GtkWidget *font_label;
-          
-          fontsel = gtk_font_button_new ();
-          g_object_set_data (G_OBJECT (editor), "font-selector", fontsel);
+      fontsel = gtk_font_button_new ();
+      g_object_set_data (G_OBJECT (editor), "font-selector", fontsel);
 
-          gtk_font_button_set_title (GTK_FONT_BUTTON (fontsel), _("Choose A Terminal Font"));
-          gtk_font_button_set_show_size (GTK_FONT_BUTTON (fontsel), TRUE);
-          gtk_font_button_set_show_style (GTK_FONT_BUTTON (fontsel), FALSE);
-          gtk_font_button_set_use_font (GTK_FONT_BUTTON (fontsel), TRUE);
-          gtk_font_button_set_use_size (GTK_FONT_BUTTON (fontsel), FALSE);
+      gtk_font_button_set_title (GTK_FONT_BUTTON (fontsel), _("Choose A Terminal Font"));
+      gtk_font_button_set_show_size (GTK_FONT_BUTTON (fontsel), TRUE);
+      gtk_font_button_set_show_style (GTK_FONT_BUTTON (fontsel), FALSE);
+      gtk_font_button_set_use_font (GTK_FONT_BUTTON (fontsel), TRUE);
+      gtk_font_button_set_use_size (GTK_FONT_BUTTON (fontsel), FALSE);
 
-          profile_editor_update_font (editor, profile);
-          g_signal_connect (G_OBJECT (fontsel), "font_set",
-                            G_CALLBACK (font_set),
-                            profile);
+      profile_editor_update_font (editor, profile);
+      g_signal_connect (G_OBJECT (fontsel), "font_set",
+			G_CALLBACK (font_set),
+			profile);
 
-          font_label = gtk_label_new_with_mnemonic (_("_Font:"));
-          gtk_misc_set_alignment (GTK_MISC (font_label), 0.0, 0.5);
-          gtk_label_set_mnemonic_widget (GTK_LABEL (font_label), fontsel);
+      font_label = gtk_label_new_with_mnemonic (_("_Font:"));
+      gtk_misc_set_alignment (GTK_MISC (font_label), 0.0, 0.5);
+      gtk_label_set_mnemonic_widget (GTK_LABEL (font_label), fontsel);
 
-          gtk_box_set_spacing (GTK_BOX (w), 12);
-          
-          gtk_box_pack_start (GTK_BOX (w), GTK_WIDGET (font_label), FALSE, FALSE, 0);
-          gtk_box_pack_start (GTK_BOX (w), GTK_WIDGET (fontsel), FALSE, FALSE, 0);
-
-          size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-          gtk_size_group_add_widget (size_group,
-                                     font_label);
-          gtk_size_group_add_widget (size_group,
-                                     glade_xml_get_widget (xml,
-                                                           "profile-name-label"));
-          gtk_size_group_add_widget (size_group,
-                                     glade_xml_get_widget (xml,
-                                                           "profile-icon-label"));
-          g_object_unref (G_OBJECT (size_group));
-        }
-      else
-        {
-          fontsel = egg_xfont_selector_new (_("Choose A Terminal Font"));
-          g_object_set_data (G_OBJECT (editor), "font-selector", fontsel);
- 
-          profile_editor_update_x_font (editor, profile);
+      gtk_box_set_spacing (GTK_BOX (w), 12);
       
-          g_signal_connect (fontsel, "changed",
-                            G_CALLBACK (font_changed), profile);
+      gtk_box_pack_start (GTK_BOX (w), GTK_WIDGET (font_label), FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (w), GTK_WIDGET (fontsel), FALSE, FALSE, 0);
 
-          size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-          gtk_size_group_add_widget (size_group,
-                                     EGG_XFONT_SELECTOR (fontsel)->family_label);
-          gtk_size_group_add_widget (size_group,
-                                     glade_xml_get_widget (xml,
-                                                           "profile-name-label"));
-          gtk_size_group_add_widget (size_group,
-                                     glade_xml_get_widget (xml,
-                                                           "profile-icon-label"));
-          g_object_unref (G_OBJECT (size_group));
-
-          gtk_box_pack_start (GTK_BOX (w), GTK_WIDGET (fontsel), TRUE, TRUE, 0);
-        }
+      size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+      gtk_size_group_add_widget (size_group,
+				 font_label);
+      gtk_size_group_add_widget (size_group,
+				 glade_xml_get_widget (xml,
+						       "profile-name-label"));
+      gtk_size_group_add_widget (size_group,
+				 glade_xml_get_widget (xml,
+						       "profile-icon-label"));
+      g_object_unref (G_OBJECT (size_group));
       
       gtk_widget_show_all (w);
       
@@ -1308,6 +1279,21 @@ terminal_profile_edit (TerminalProfile *profile,
   gtk_window_present (GTK_WINDOW (editor));
 }
 
+static void
+widget_and_labels_set_sensitive (GtkWidget *widget, gboolean sensitive)
+{
+  GList *labels, *i;
+
+  labels = gtk_widget_list_mnemonic_labels (widget);
+  for (i = labels; i; i = i->next)
+    {
+      gtk_widget_set_sensitive (GTK_WIDGET (i->data), sensitive);
+    }
+  g_list_free (labels);
+
+  gtk_widget_set_sensitive (widget, sensitive);
+}
+
 static void
 set_insensitive (GtkWidget  *editor,
                  const char *widget_name,
@@ -1317,7 +1303,7 @@ set_insensitive (GtkWidget  *editor,
 
   w = profile_editor_get_widget (editor, widget_name);
 
-  gtk_widget_set_sensitive (w, !setting);
+  widget_and_labels_set_sensitive (w, !setting);
 }
 
 static void
@@ -1343,10 +1329,9 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
   /* This one can't be short-circuited by the cache, since
    * it depends on settings
    */
-  w = profile_editor_get_widget (editor, "custom-command-entry");
-  gtk_widget_set_sensitive (w,
-                            !((mask->custom_command) ||
-                              !terminal_profile_get_use_custom_command (profile))); 
+  set_insensitive (editor, "custom-command-entry",
+                   mask->custom_command ||
+                     !terminal_profile_get_use_custom_command (profile));
 
   w = profile_editor_get_widget (editor, "custom-command-entry-label");
   gtk_widget_set_sensitive (w,
@@ -1354,50 +1339,30 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
                               !terminal_profile_get_use_custom_command (profile))); 
   if (terminal_profile_get_background_type (profile) == TERMINAL_BACKGROUND_IMAGE)
     {
-      w = profile_editor_get_widget (editor, "background-image-filechooser");
-      gtk_widget_set_sensitive (w, !(mask->background_image_file));
-      w = profile_editor_get_widget (editor, "background-image-filechooser-label");
-      gtk_widget_set_sensitive (w, !(mask->background_image_file));
-      w = profile_editor_get_widget (editor, "scroll-background-checkbutton");
-      gtk_widget_set_sensitive (w, !(mask->scroll_background));
-      w = profile_editor_get_widget (editor, "darken-background-vbox");
-      gtk_widget_set_sensitive (w, !(mask->background_darkness));
+      set_insensitive (editor, "background-image-filechooser", mask->background_image_file);
+      set_insensitive (editor, "scroll-background-checkbutton", mask->scroll_background);
+      set_insensitive (editor, "darken-background-vbox", mask->background_darkness);
     }
   else if (terminal_profile_get_background_type (profile) == TERMINAL_BACKGROUND_TRANSPARENT) 
-  {
-      w = profile_editor_get_widget (editor, "darken-background-vbox");
-      gtk_widget_set_sensitive (w, !(mask->background_darkness));
-      w = profile_editor_get_widget (editor, "background-image-filechooser");
-      gtk_widget_set_sensitive (w, FALSE);
-      w = profile_editor_get_widget (editor, "background-image-filechooser-label");
-      gtk_widget_set_sensitive (w, FALSE);
-      w = profile_editor_get_widget (editor, "scroll-background-checkbutton");
-      gtk_widget_set_sensitive (w, FALSE);
-      
+    {
+      set_insensitive (editor, "background-image-filechooser", TRUE);
+      set_insensitive (editor, "scroll-background-checkbutton", TRUE);
+      set_insensitive (editor, "darken-background-vbox", mask->background_darkness);
     }
   else
     {
-      w = profile_editor_get_widget (editor, "background-image-filechooser");
-      gtk_widget_set_sensitive (w, FALSE);
-      w = profile_editor_get_widget (editor, "background-image-filechooser-label");
-      gtk_widget_set_sensitive (w, FALSE);
-      w = profile_editor_get_widget (editor, "scroll-background-checkbutton");
-      gtk_widget_set_sensitive (w, FALSE);
-      w = profile_editor_get_widget (editor, "darken-background-vbox");
-      gtk_widget_set_sensitive (w, FALSE);
+      set_insensitive (editor, "background-image-filechooser", TRUE);
+      set_insensitive (editor, "scroll-background-checkbutton", TRUE);
+      set_insensitive (editor, "darken-background-vbox", TRUE);
     }
 
   if (!terminal_profile_get_use_system_font (profile))
     {
-      if (terminal_widget_supports_pango_fonts ())
-        set_insensitive (editor, "font-hbox", mask->font);
-      else
-        set_insensitive (editor, "font-hbox", mask->x_font);
+      set_insensitive (editor, "font-hbox", mask->font);
     }
   else
     {
-      w = profile_editor_get_widget (editor, "font-hbox");
-      gtk_widget_set_sensitive (w, FALSE);
+      set_insensitive (editor, "font-hbox", TRUE);
     }
 
 
@@ -1622,22 +1587,20 @@ profile_editor_update_color_scheme_menu (GtkWidget       *editor,
                                          TerminalProfile *profile)
 {
   GdkColor fg, bg;
-  int i;
+  unsigned int i;
   GtkWidget *w;
 
   w = profile_editor_get_widget (editor, "color-scheme-combobox");
   
   terminal_profile_get_color_scheme (profile, &fg, &bg);
 
-  i = 0;
-  while (i < G_N_ELEMENTS (color_schemes))
+  for (i = 0; i < G_N_ELEMENTS (color_schemes); i++)
     {
       if (gdk_color_equal (&color_schemes[i].foreground,
                            &fg) &&
           gdk_color_equal (&color_schemes[i].background,
                            &bg))
         break;
-      ++i;
     }
 
   /* If we didn't find a match, then we want the last combo
@@ -1850,27 +1813,23 @@ profile_editor_update_palette (GtkWidget       *editor,
                                TerminalProfile *profile)
 {
   GtkWidget *w;
-  int i;
+  unsigned int i;
   GdkColor palette[TERMINAL_PALETTE_SIZE];
 
   terminal_profile_get_palette (profile, palette);
   
-  i = 0;
-  while (i < TERMINAL_PALETTE_SIZE)
+  for (i = 0; i < TERMINAL_PALETTE_SIZE; i++)
     {
       char *s = g_strdup_printf ("palette-colorpicker-%d", i+1);
       w = profile_editor_get_widget (editor, s);
       g_free (s);
       
       colorpicker_set_if_changed (w, &palette[i]);
-      
-      ++i;
     }
 
   w = profile_editor_get_widget (editor, "palette-combobox");
 
-  i = 0;
-  while (i < G_N_ELEMENTS (palette_schemes))
+  for (i = 0; i < G_N_ELEMENTS (palette_schemes); i++)
     {
       int j;
       gboolean match;
@@ -1891,8 +1850,6 @@ profile_editor_update_palette (GtkWidget       *editor,
 
       if (match)
         break;
-      
-      ++i;
     }
 
   /* If we didn't find a match, then we want the last combo
@@ -1901,51 +1858,6 @@ profile_editor_update_palette (GtkWidget       *editor,
   gtk_combo_box_set_active (GTK_COMBO_BOX (w), i);
 }
 
-static void
-profile_editor_update_x_font (GtkWidget       *editor,
-                              TerminalProfile *profile)
-{
-  GtkWidget *fontsel;
-  char *spacings[] = { "m", "c", NULL };
-  char *slants[] = { "r", "ot", NULL };
-  char *weights[] = { "medium", "regular", "demibold", NULL };
-  gchar *name;
-
-  if (terminal_widget_supports_pango_fonts ())
-    return;
-  
-  fontsel = g_object_get_data (G_OBJECT (editor), "font-selector");
-
-  /* If the current selector font is the same as the new font,
-   * don't do any work.
-   */
-  name = egg_xfont_selector_get_font_name (EGG_XFONT_SELECTOR (fontsel));
-  
-  if (name == NULL)
-    return;
-
-  if (strcmp (name, terminal_profile_get_x_font (profile)) == 0)
-    {
-      g_free (name);
-      return;
-    }
-  
-  g_free (name);
-  
-  egg_xfont_selector_set_font_name (EGG_XFONT_SELECTOR (fontsel),
-				    terminal_profile_get_x_font (profile));
-
-  egg_xfont_selector_set_filter (EGG_XFONT_SELECTOR (fontsel),
-				 EGG_XFONT_FILTER_BASE,
-				 EGG_XFONT_ALL,
-				 NULL,
-				 weights,
-				 slants,
-				 NULL,
-				 spacings,
-				 NULL);
-}
-
 static void
 profile_editor_update_background_type (GtkWidget       *editor,
                                        TerminalProfile *profile)
@@ -1966,6 +1878,8 @@ profile_editor_update_background_type (GtkWidget       *editor,
       w = profile_editor_get_widget (editor, "transparent-radiobutton");
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
       break;
+    default:
+      g_assert_not_reached ();
     }
 }
 
@@ -1974,11 +1888,13 @@ profile_editor_update_background_image (GtkWidget       *editor,
                                         TerminalProfile *profile)
 {
   GtkWidget *w;
+  const char *f;
 
   w = profile_editor_get_widget (editor, "background-image-filechooser");
 
-  gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (w),
-                                 terminal_profile_get_background_image_file (profile));
+  f = terminal_profile_get_background_image_file (profile);
+  if (f && *f)
+    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (w), f);
 }
 
 static void
@@ -2061,9 +1977,6 @@ profile_editor_update_font (GtkWidget       *editor,
 {
   GtkWidget *w;
 
-  if (!terminal_widget_supports_pango_fonts ())
-    return;
-  
   w = g_object_get_data (G_OBJECT (editor), "font-selector");
 
   fontpicker_set_if_changed (w,
