@@ -56,7 +56,6 @@ typedef struct
   unsigned int icon_file : 1;
   unsigned int is_default : 1;
   unsigned int palette : 1;
-  unsigned int x_font : 1;
   unsigned int background_type : 1;
   unsigned int background_image_file : 1;
   unsigned int scroll_background : 1;
@@ -173,7 +172,6 @@ GdkPixbuf*                terminal_profile_get_icon                 (TerminalPro
 gboolean                  terminal_profile_get_is_default           (TerminalProfile *profile);
 void                      terminal_profile_get_palette              (TerminalProfile *profile,
                                                                      GdkColor        *colors);
-const char*               terminal_profile_get_x_font               (TerminalProfile *profile);
 
 TerminalBackgroundType terminal_profile_get_background_type       (TerminalProfile *profile);
 GdkPixbuf*             terminal_profile_get_background_image      (TerminalProfile *profile);
@@ -237,8 +235,6 @@ void terminal_profile_set_palette              (TerminalProfile *profile,
 void terminal_profile_set_palette_entry        (TerminalProfile *profile,
                                                 int              i,
                                                 const GdkColor  *color);
-void terminal_profile_set_x_font               (TerminalProfile *profile,
-                                                const char      *name);
 
 void terminal_profile_set_background_type       (TerminalProfile        *profile,
                                                  TerminalBackgroundType  type);
@@ -282,9 +278,9 @@ const TerminalSettingMask* terminal_profile_get_locked_settings (TerminalProfile
 
 void terminal_profile_update (TerminalProfile *profile);
 
-void terminal_profile_create (TerminalProfile *base_profile,
-                              const char      *visible_name,
-                              GtkWindow       *transient_parent);
+char* terminal_profile_create (TerminalProfile *base_profile,
+                               const char      *visible_name,
+                               GtkWindow       *transient_parent);
 
 void terminal_profile_delete_list (GConfClient *conf,
                                    GList      *list,
@@ -299,6 +295,7 @@ gboolean terminal_setting_mask_equal    (const TerminalSettingMask *a,
 extern const GdkColor terminal_palette_linux[TERMINAL_PALETTE_SIZE];
 extern const GdkColor terminal_palette_xterm[TERMINAL_PALETTE_SIZE];
 extern const GdkColor terminal_palette_rxvt[TERMINAL_PALETTE_SIZE];
+extern const GdkColor terminal_palette_tango[TERMINAL_PALETTE_SIZE];
 
 char*    terminal_palette_to_string   (const GdkColor *palette);
 gboolean terminal_palette_from_string (const char     *str,
